﻿using HighWayIot.Log4net;
using HighWayIot.Repository.domain;
using Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace HighWayIot.Repository.service
{
    public class ZxRecipePositionParaService
    {
        private static readonly Lazy<ZxRecipePositionParaService> lazy = new Lazy<ZxRecipePositionParaService>(() => new ZxRecipePositionParaService());

        public static ZxRecipePositionParaService Instance
        {
            get
            {
                return lazy.Value;
            }
        }

        private LogHelper log = LogHelper.Instance;
        Repository<ZxRecipePositionParaEntity> _repository => new Repository<ZxRecipePositionParaEntity>("sqlserver");

        /// <summary>
        /// 查询工位配方字段信息
        /// </summary>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfos()
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList();
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息获取异常", ex);
                return null;
            }
        }

        /// <summary>
        /// 查询工位配方字段信息
        /// </summary>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfos(Expression<Func<ZxRecipePositionParaEntity, bool>> whereExpression)
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList(whereExpression);
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息获取异常", ex);
                return null;
            }
        }


        /// <summary>
        /// 根据配方号查询工位配方字段信息
        /// </summary>
        /// <param name="id">工位配方编号</param>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfoByRecipeCode(string recipeCode)
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList(x => x.RecipeCode == recipeCode);
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息获取异常", ex);
                return new List<ZxRecipePositionParaEntity>();
            }
        }

        /// <summary>
        /// 修改工位配方字段信息
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool UpdateRecipePositionParaInfo(ZxRecipePositionParaEntity entity)
        {
            try
            {
                return _repository.Update(entity);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息修改异常", ex);
                return false;
            }
        }

        /// <summary>
        /// 添加工位配方字段信息
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool InsertRecipePositionParaInfo(ZxRecipePositionParaEntity entity)
        {
            try
            {
                return _repository.Insert(entity);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息修改异常", ex);
                return false;
            }
        }

        /// <summary>
        /// ID删除工位配方字段信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool DeleteRecipePositionParaInfoById(int id)
        {
            try
            {
                return _repository.DeleteById(id);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息删除异常", ex);
                return false;
            }
        }

        /// <summary>
        /// ID删除工位配方字段信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool DeleteRecipePositionParaInfoByRecipeCode(string recipeCode)
        {
            try
            {
                return _repository.Delete(x => x.RecipeCode == recipeCode);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息删除异常", ex);
                return false;
            }
        }

        /// <summary>
        /// 清除脏数据
        /// </summary>
        /// <returns></returns>
        public bool DeleteDirtyData()
        {
            try
            {
                var entities = _repository.GetList();

                entities = _repository.GetList().Where(x => x.Position == null || (x.RecipeCode == null || x.RecipeCode == string.Empty)).ToList();

                foreach (var entity in entities)
                {
                    if (!DeleteRecipePositionParaInfoById(entity.Id))
                    {
                        return false;
                    }
                }

                entities = _repository.GetList();

                foreach(var e in entities.GroupBy(x => x.RecipeCode).ToList())
                {
                    foreach(var en in e.GroupBy(x => x.Position).ToList())
                    {
                        var l = en.ToList();
                        if(l.Count > 1)
                        {
                            l.Remove(l.Where(x => x.Id == l.Min(y => y.Id)).FirstOrDefault());
                            foreach (var entity in l)
                            {
                                if (!DeleteRecipePositionParaInfoById(entity.Id))
                                {
                                    return false;
                                }
                            }
                        }
                    }
                }



                return true;
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息删除异常", ex);
                return false;
            }
        }
    }
}
