﻿using HighWayIot.Log4net;
using Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HighWayIot.Repository.domain;

namespace HighWayIot.Repository.service
{

    public class ZxRecipePositionParaService
    {
        private static readonly Lazy<ZxRecipePositionParaService> lazy = new Lazy<ZxRecipePositionParaService>(() => new ZxRecipePositionParaService());

        public static ZxRecipePositionParaService Instance
        {
            get
            {
                return lazy.Value;
            }
        }

        private LogHelper log = LogHelper.Instance;
        Repository<ZxRecipePositionParaEntity> _repository => new Repository<ZxRecipePositionParaEntity>("sqlserver");

        /// <summary>
        /// 查询工位配方字段信息
        /// </summary>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfos()
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList();
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方信息获取异常", ex);
                return null;
            }
        }

        /// <summary>
        /// 查询工位配方字段信息
        /// </summary>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfoByRecipeCode()
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList();
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方信息获取异常", ex);
                return null;
            }
        }

        /// <summary>
        /// 根据工位配方号查询工位配方字段信息
        /// </summary>
        /// <param name="recipeCode">工位配方编号</param>
        /// <returns></returns>
        public List<ZxRecipePositionParaEntity> GetRecipePositionParaInfoByRecipeCode(int positionCode)
        {
            try
            {
                List<ZxRecipePositionParaEntity> entity = _repository.GetList(x => x.ParaCode == positionCode);
                return entity;
            }
            catch (Exception ex)
            {
                log.Error("工位配方信息获取异常", ex);
                return null;
            }
        }

        /// <summary>
        /// 修改工位配方字段信息
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool UpdateRecipePositionParaInfo(ZxRecipePositionParaEntity entity)
        {
            try
            {
                return _repository.Update(entity);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息修改异常", ex);
                return false;
            }
        }

        /// <summary>
        /// 添加工位配方字段信息
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool InsertRecipePositionParaInfo(ZxRecipePositionParaEntity entity)
        {
            try
            {
                return _repository.Insert(entity);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息修改异常", ex);
                return false;
            }
        }

        /// <summary>
        /// ID删除工位配方字段信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool DeleteRecipePositionParaInfoById(int id)
        {
            try
            {
                return _repository.DeleteById(id);
            }
            catch (Exception ex)
            {
                log.Error("工位配方字段信息删除异常", ex);
                return false;
            }
        }

    }
}
